package z08;

import java.io.*;
import java.util.StringTokenizer;

public class MSTdiameter {

static int NIL = -1; // null pointer

static int N, N1, N2, p1, p2;  // graph parameters
static int [] pred;       // as in standard Prim alg 
static int [] dist;       // as in standard Prim alg
static boolean [] closed; // as in standard Prim alg

static int [] treeNodeDegree;  // tree represents MST  
static int [][] tree;          // usual linked representation
static int treeDiameter = 0; 


static int edgeWeight(int n1, int n2) {
  if (n1 < n2)
    return (int)( ( ((long)n1 + N1+ (n2*(n2-1))/2 ) * p1) % p2); 
  else
    return (int)( ( ((long)n2 + N1+ (n1*(n1-1))/2 ) * p1) % p2); 
}


static void MST() {
  int minDistNeighbour, minDist, edgeWeight;
  
  // initialize search
  for(int i = 0; i < N; i++) {
    dist[i] = Integer.MAX_VALUE;
    pred[i] = NIL;
    closed[i] = false;
  }
  int currNode = 0;  // let 0 be the the root of the tree
  dist[currNode] = 0;
  
  // main search loop
  for(int i = 0; i < N-1; i++) {
    // recalc the distance and predecessor of neighbours 
    // of node currently added to the MST
    for(int neighbour = 1; neighbour < N; neighbour++) { 
      if ((closed[neighbour]) || (neighbour == currNode)) continue;
      edgeWeight =  edgeWeight(currNode,neighbour);  
      if (edgeWeight < dist[neighbour]) {
        dist[neighbour] = edgeWeight;
        pred[neighbour] = currNode;
      }
    }
    closed[currNode] = true;

    // find closest open node to be added to the MST
    minDist = Integer.MAX_VALUE;
    minDistNeighbour = NIL;
    for(int k = 1; k < N; k++)   // 0 is the root
      if (!closed[k] && (dist[k] < minDist)) {
        minDist = dist[k];
        minDistNeighbour = k;
      }
    currNode = minDistNeighbour;
  } // search loop
}


// in the tree computed in MST, the edges are pointing upwards to parents
// we must reverse their direction so that they point downwards to children
// this change enables easy recursive processing (finding height) of subtrees
static void reverseTree() {
  for(int i = 1; i < N; i++)       
    treeNodeDegree[pred[i]]++;  
  for(int i = 0; i < N; i++) { 
    tree[i] = new int [treeNodeDegree[i]];
     treeNodeDegree[i] = 0;   
  }
  for(int i = 1; i < N; i++)    
    tree[pred[i]][treeNodeDegree[pred[i]]++] = i;
}

static int heightAndDiam(int node) {
  if (treeNodeDegree[node] == 0) return 0;  //leaf, stop recursion 
  
  int subheight1=-1;  // biggest height of subtree
  int subheight2=-1;  // second biggest height of subtree
  int subheight;      // just a variable
  
  for(int j = 0; j < treeNodeDegree[node]; j++) {
    subheight = heightAndDiam(tree[node][j]);     // child height
    if (subheight > subheight1) 
      { subheight2 = subheight1; subheight1 = subheight; }
    else if (subheight > subheight2)
      subheight2 = subheight;
    }
  if (subheight1+subheight2 +2 > treeDiameter) 
    treeDiameter = subheight1+subheight2 +2;
  return subheight1+1;
}

public static void main(String[] args) throws IOException {
  long t1, t2, t3,t4, t5;  // time measurement
  
  // read input
  BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
  StringTokenizer st = new StringTokenizer(br.readLine());
  N1 = Integer.valueOf(st.nextToken());
  N2 = Integer.valueOf(st.nextToken());
  N = N2-N1+1;
  p1 = Integer.valueOf(st.nextToken());
  p2 = Integer.valueOf(st.nextToken());
  
  // allocate the arrays
        t1 = System.currentTimeMillis();
  pred = new int [N];
  dist = new int [N];
  closed = new boolean[N];
  treeNodeDegree = new int [N];
  tree = new int [N][];
  
  // solve
        t2 = System.currentTimeMillis();  
  MST(); 
        t3 = System.currentTimeMillis();
  reverseTree();
        t4 = System.currentTimeMillis();
  heightAndDiam(0);
        t5 = System.currentTimeMillis();

  // results
  System.out.printf("tree diameter %d\n", treeDiameter);  
  System.out.printf("time alloc %d\n", t2-t1);
  System.out.printf("time MST   %d\n", t3-t2);
  System.out.printf("time rever %d\n", t4-t3);
  System.out.printf("time heigh %d\n", t5-t4);
  System.out.printf("time TOTAL %d\n", t5-t1);
}


}
